// This is my sketch. 
// It is free for you to use and modify as you please.
// As it is my first sketch, please be lenient with it.
// The sketch is meant to use a Arduino Nano, together 
// with a LCD-Display and a sensor board to let the CO2 ppm 
// concentration, the temperature and Humidity.
// It also has a rudimentary timer to remind you to open the 
// window from time to time.
// if you like it - great! If you don't - too bad.
// Please tell me your thoughts and suggestions on Instructables:
// 

// 0 This is where you tell the compiler which libraries are needed
//   SPI.h and Wire.h are included, because of the LCD display and the sensor chip and cause I'd like to include I2C reports - some day.... 
#include <SPI.h>
#include <Wire.h>
#include "ccs811.h"                             // 1 for the CO2 sensor
#include "ClosedCube_HDC1080.h"                 // 2 for the temperature and humidity sensor
#include <LiquidCrystal_I2C.h>                  // 3 for the LCD display

LiquidCrystal_I2C LCD(39, 20, 4);               // 4 Defines "LCD" as an object with I3C address "39", 20 characters/line and 4 lines
CCS811 ccs811(13);                              // 5 Defines "css811" as an object, that gets a wake-up signal (WAK) from the Nano Pin 13
ClosedCube_HDC1080 hdc1080;                     // 6 Defines the hdc sensor as an object "hdc1080"

// Next we define some variables and their starting values
int timer = 0;                                  // 6 Variable, Integer needed to count seconds for my "timer" routine
bool air = 0;                                   // 7 Variable, Boolean Yes/No, True/False, 1/0 variable to check when to open/close the window
int Buzzer = 5;                                 // 8 Variable, Integer needed to tell the Nano which pin the buzzer is on
int RED = 6;                                    // 9 Variable, Integer needed to tell the Nano which pins the LEDs are on
int YELLOW = 7;                                 // 9 Variable, Integer needed to tell the Nano which pins the LEDs are on
int GREEN = 8;                                  // 9 Variable, Integer needed to tell the Nano which pins the LEDs are on

uint16_t temperaturecorrection = 6.5;           // 10 Corrects the temperature reading of the chip to what you measured yourself
                                                // 10 The CCS811 gives off temperature - so change accordingly

void setup() {                                  // 11 Lets start up the processor....

  // 12 Enable the serial connection and say what to print on your screen in the IDE under Tools/SerialManitor
  //    This is only here to show you that it's possible
  Serial.begin(115200);
  Serial.println();
  Serial.println("setup: Starting CCS811");
  Serial.print("setup: ccs811 lib  version: "); 
  Serial.println(CCS811_VERSION);

  // 13 Enable I2C so you can use the LCD and sensor board
  Wire.begin();

  // 14 Warm up The LCD display and show some initial Messages
  LCD.init();
  LCD.clear();
  LCD.backlight();
  LCD.setCursor(1, 1);
  LCD.print("Hallo, Mona!");                    // my wife ;-)
  LCD.setCursor(4, 2);
  LCD.print("Hallo, Alle!");                    // we're in Germany
  delay(5000);                                  // Wait 5 seconds before erasing
  LCD.clear();
  
  // 15 activate the CCS811 sensor
  ccs811.set_i2cdelay(50);                     // Needed for ESP8266 because it doesn't handle I2C clock stretch correctly
  bool ok= ccs811.begin();                     // Check if it started OK and return errors, if not
  if( !ok ) { LCD.setCursor(0, 1); LCD.print("setup: CCS811 NOK"); }
  if( ok ) { LCD.setCursor(0, 1); LCD.print("setup: CCS811 OK"); }
  delay(1000);                                 // Wait a second, before clearing the display
  LCD.clear();
  
  // 16 activate the HDC1080 on the sensor board
  hdc1080.begin(0x40);

  // 17 Tell the Nano to take the value of the variable you defined and thus define the pin # as output-pin
  pinMode(Buzzer, OUTPUT);                     
  pinMode(RED, OUTPUT);
  pinMode(YELLOW, OUTPUT);
  pinMode(GREEN, OUTPUT);
   
  // 18 If the sensor works, start measuring, else diesplay an error....
  ok= ccs811.start(CCS811_MODE_1SEC);
  if( !ok ) { LCD.setCursor(0, 1); LCD.print("setup: CCS811 NOK"); 
  delay(5000);
  LCD.clear();  
  }
}                                               // That was all only preliminary work

void loop() {                                   // 19 Let's get going.....

  // 20 Read some data from your sensors. 
  uint16_t eco2, tvoc, errstat, raw;                          // 21 set a couple of variables
  float temperatureread = hdc1080.readTemperature();          // 22 Define a Variable, read the temperature from the object with the definition from its library
  float temperature = temperatureread - temperaturecorrection; // 23 Correct the temperature with the correction you measured yourself
  float humidity = hdc1080.readHumidity();                    // 24 Define a variable, read  the humidity from the sensor
  ccs811.set_envdata(temperature, humidity);                  // 25 Tell the cc811 sensor the temperature and humidity
  ccs811.read(&eco2, &tvoc, &errstat, &raw);                  // 26 Read the values for the variables you defined above from the sensor

  // 27 Here is where you display the data you collected on the LCD display 
  LCD.clear(); 
  // 28 Either define the new curser-position each time and then print the information, or use stings to put the information together ant print as one:
  //LCD.setCursor(0, 0); LCD.print("Temperature: "); LCD.setCursor(12, 0); LCD.print(temperature); LCD.setCursor(18, 0); LCD.print("C");
  LCD.setCursor(0, 0); LCD.print(String("Temperatur: ") + String(temperature) + String(" C"));
  LCD.setCursor(0, 1); LCD.print(String("Luftfeuchte: ") + String(humidity) + String(" %"));
  LCD.setCursor(0, 2); LCD.print(String("eCO2: ") + String(eco2) + String(" ppm"));
  LCD.setCursor(0, 3); LCD.print(String("TVOC: ") + String(tvoc) + String(" ppb"));

  // 29 Let's check if we're still safe and show it on the LEDs with simple check, by activating the pins you defined as output
  if (eco2 <= 1000){
    digitalWrite(RED, LOW);
    digitalWrite(YELLOW, LOW);
    digitalWrite(GREEN, HIGH);
  }
  else if (eco2 >1000 and eco2 <= 2000){
    digitalWrite(RED, LOW);
    digitalWrite(YELLOW, HIGH);
    digitalWrite(GREEN, LOW);    
  }

  else if (eco2 > 2000) {
     digitalWrite(RED, HIGH);
     digitalWrite(YELLOW, LOW);
     digitalWrite(GREEN, LOW);   
  }  
 

  // 30 A simple timer, that counts to 20min, beeps, counts to 5 min, beeps, and so on and displays what you should do....
  // wait 20 min until air = 1200s, but since Loop every 2 s, only 600s
  if (timer < 600 and !air) {
    LCD.setCursor(15, 2); LCD.print("Zu   ");
    LCD.setCursor(14, 3); LCD.print(String(timer/30) + String(" min"));
  }
  if (timer == 600 and !air) {
    air = 1;
    timer = 0;
    LCD.setCursor(15, 2); LCD.print("AUF!!");
    LCD.setCursor(15, 3); LCD.print(String(timer) + String("min"));
    digitalWrite(Buzzer, HIGH); 
    delay(3000);
    digitalWrite(Buzzer, LOW);
  }
  if (timer < 150 and air) {
    LCD.setCursor(15, 2); LCD.print("AUF!!");
    LCD.setCursor(14, 3); LCD.print(String(timer/30) + String(" min"));    
  }
  if (timer == 150 and air) {
    air = 0;
    timer = 0;
    LCD.setCursor(15, 2); LCD.print("Zu   ");
    LCD.setCursor(14, 3); LCD.print(String(timer) + String(" min"));
    digitalWrite(Buzzer, HIGH); 
    delay(3000);
    digitalWrite(Buzzer, LOW);
  }
  timer = timer + 1;
 

//Wait 2 seconds
delay(2000);

}   // 31 That's it! Let's start a new round....
